
from odoo import api, models
from odoo.addons.saas_worker import metabase
from odoo.addons.saas_worker.const import TEST
from odoo.addons.print_docsaway.data.print_docsaway_error import _DOCSAWAY_ERROR_MAIL

ODOO_MAIL = 'administrator@odoo.com'
ODOO_KEY = '76qF8iub0hX2dhOVtU55bo9X2l7jWp1FXHwt6UVh3MLbsliA0sNZ5GGoKyJ5hR0I'
ODOO_ENV = 'TEST' if TEST else 'LIVE'  # 'LIVE' or 'TEST' for docsaway provider environement


class PrintOrder(models.Model):

    _inherit = 'print.order'

    @api.model
    def create(self, values):
        order = super(PrintOrder, self).create(values)
        if order.provider_id and order.provider_id.provider == 'docsaway':
            # increment the print waiting counter of the metabase
            db = metabase.get(self._cr.dbname)
            if db.print_counter + db.print_waiting_counter < db.print_counter_limit:
                metabase.update(db, print_waiting_counter=db.print_waiting_counter+1)
        return order

    @api.multi
    def write(self, values):
        """ if the state changed, the print counter of metabase may change too """
        new_state = values.get('state')
        counter = 0
        db = metabase.get(self._cr.dbname)
        if new_state:
            waiting_counter = db.print_waiting_counter
            # count the number of order that can be free (include in the 'waiting_counter') and that are done.
            for order in self:
                old_state = order.state
                if old_state in ['draft', 'ready', 'error'] and new_state in ['sent'] and waiting_counter - counter > 0:
                    counter += 1
        # execute write
        result = super(PrintOrder, self).write(values)
        # mark the transfert of order into the 'sent' state : convert them from waiting to finished
        # update metabase after write, so if write raise error, metabase not updated
        if counter:
            metabase.update(db, print_waiting_counter=db.print_waiting_counter-counter, print_counter=db.print_counter+counter)
        return result

    @api.multi
    def unlink(self):
        waiting_orders = len(self.filtered(lambda order: order.state in ['draft', 'ready', 'error']))
        result = super(PrintOrder, self).unlink()
        if waiting_orders:
            db = metabase.get(self._cr.dbname)
            new_waiting_counter = db.print_waiting_counter - waiting_orders if db.print_waiting_counter > waiting_orders else 0
            metabase.update(db, print_waiting_counter=new_waiting_counter)
        return result


class PrintProvider(models.Model):

    _inherit = 'print.provider'

    def get_free_remaining_letters(self):
        remaining = 0
        db = metabase.get(self._cr.dbname)
        if db.mode == 'paid':  # free printings only for paid databases
            user_counter = db.print_counter + db.print_waiting_counter
            remaining = db.print_counter_limit - user_counter if db.print_counter_limit > user_counter else 0
        return remaining

    def get_free_waiting_letters(self):
        waiting = 0
        db = metabase.get(self._cr.dbname)
        if db.mode == 'paid':
            waiting = db.print_waiting_counter
        return waiting

    # --------------------------------------------------
    # Methods to be redefined from print.provider
    # Prevent raising exception if client can use Odoo Docasaway Account
    # --------------------------------------------------
    def docsaway_check_configuration(self):
        """ Don't care about the config of the user provider if he'll get free printing from Odoo """
        if self.get_free_remaining_letters() > 0:
            return True
        return super(PrintProvider, self).docsaway_check_configuration()

    def docsaway_check_credit(self):
        """ Don't care about the credit of the user provider if he'll get free printing from Odoo """
        if self.get_free_remaining_letters() > 0:
            return True
        return super(PrintProvider, self).docsaway_check_credit()

    # --------------------------------------------------
    # DocsAway API methods
    # --------------------------------------------------
    @api.model
    def _docsaway_get_credentials(self):
        """ Return Odoo Credentials instead of client's ones if the client
            still have access to free printings
        """
        if self._context.get('print_free_odoo'):
            return ODOO_MAIL, ODOO_KEY
        return super(PrintProvider, self)._docsaway_get_credentials()

    def _docsaway_do_server_request(self, url, request, error_map=_DOCSAWAY_ERROR_MAIL):
        """ Force the APIMode in LIVE (not in TEST) if free printing available """
        if 'APIMode' in request and self._context.get('print_free_odoo'):  # force ODOO_ENV if free
            request['APIMode'] = ODOO_ENV
        return super(PrintProvider, self)._docsaway_do_server_request(url, request, error_map=error_map)

    # NOTE : _docsaway_fetch_account make the update of docsaway user account. It must use the user credentials.
    # So don't override _docsaway_fetch_account method
    # But fetching station (_docsaway_fetch_station), fetching order price (_docsaway_fetch_price) and sending
    # document (_docsaway_send_document) should pass the cntext key `print_free_odoo` if the order is offered by Odoo,
    # so if there the order is count as a 'waiting' order (metabase, field print_waiting_counter).

    def _docsaway_fetch_station(self, destination, ink='BW', paper='80', stationID='AUTO'):
        print_free_odoo = bool(self.get_free_waiting_letters() > 0)
        return super(PrintProvider, self.with_context(print_free_odoo=print_free_odoo))._docsaway_fetch_station(destination, ink=ink, paper=paper, stationID=stationID)

    def _docsaway_fetch_price(self, station, courier, zone, currency_name, nb_pages, ink='BW', paper=80):
        print_free_odoo = bool(self.get_free_waiting_letters() > 0)
        return super(PrintProvider, self.with_context(print_free_odoo=print_free_odoo))._docsaway_fetch_price(station, courier, zone, currency_name, nb_pages, ink=ink, paper=paper)

    def _docsaway_send_document(self, address, printing_infos, PDFfile):
        print_free_odoo = bool(self.get_free_waiting_letters() > 0)
        result = super(PrintProvider, self.with_context(print_free_odoo=print_free_odoo))._docsaway_send_document(address, printing_infos, PDFfile)
        return result

    # --------------------------------------------------
    # CRUD
    # --------------------------------------------------
    @api.multi
    def write(self, vals):
        for record in self:
            if record.provider == 'docsaway':
                if record.get_free_waiting_letters() > 0:
                    # Do not update these fields since the process was exectued with Odoo Credentials, otherwise
                    # saas people are gonna see our balance, account ref, ...
                    for k in ['docsaway_name', 'docsaway_reference', 'docsaway_volume', 'balance']:
                        if k in vals:
                            del vals[k]
        return super(PrintProvider, self).write(vals)
